package 
{
	import org.papervision3d.cameras.FreeCamera3D;
	import org.papervision3d.core.geom.renderables.Vertex3D;


	/**
	* Camera system that uses target, distance, pitch, and yaw. 
	* Must be used in tandem with a FreeCamera3D via "toFreeCamera3D" method.
	
	* Revision: 12/2007
	*/
	public class LeeCamera3D
	{
		
		private const DEG:Number = Math.PI / 180;
		public var target		:Vertex3D;
		public var distance		:Number;
		public var pitch		:Number;
		public var yaw			:Number;	
		
		public function LeeCamera3D($target:Vertex3D=null, $distance:Number=1000, $pitch:Number=0, $yaw:Number=0)
		{
			if ($target)	
				target = new Vertex3D($target.x, $target.y, $target.z);
			else
				target = new Vertex3D();
			distance = $distance;
			pitch = $pitch;
			yaw = $yaw;
		}
		
		public function toFreeCamera3D($freecam:FreeCamera3D):void
		{
			var p:Vertex3D = new Vertex3D(0, 0, -this.distance);
		
			p = rotX(p, this.pitch * DEG);
			p = rotY(p, this.yaw * DEG);
	
			$freecam.x = p.x + this.target.x; 
			$freecam.y = p.y + this.target.y; 
			$freecam.z = p.z + this.target.z;
			$freecam.rotationX = -this.pitch; 
			$freecam.rotationY = -this.yaw;
			$freecam.rotationZ = 0;
		} 
	
		public static function rotX(p:Vertex3D, angleZ:Number) : Vertex3D {
			var y1:Number = Math.cos(angleZ) * p.y - Math.sin(angleZ) * p.z;
			var z1:Number = Math.cos(angleZ) * p.z + Math.sin(angleZ) * p.y;
			var x1:Number = p.x;
			return new Vertex3D(x1, y1, z1);
		}
		public static function rotY(p:Vertex3D, angleY:Number) : Vertex3D {
			var x1:Number = Math.cos(angleY) * p.x - Math.sin(angleY) * p.z;
			var z1:Number = Math.cos(angleY) * p.z + Math.sin(angleY) * p.x;
			var y1:Number = p.y;
			return new Vertex3D(x1, y1, z1);
		}	
		public static function rotZ(p:Vertex3D, angleZ:Number) : Vertex3D {
			var x1:Number = Math.cos(angleZ) * p.x - Math.sin(angleZ) * p.y;
			var y1:Number = Math.cos(angleZ) * p.y + Math.sin(angleZ) * p.x;
			var z1:Number = p.z;
			return new Vertex3D(x1, y1, z1);
		}	
	}
}